// +build !windows
// errorcheckdir -n

// Copyright 2011 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package ignored

// TODO(ysmolsky): Fix golang.org/issue/25693 to enable on Windows.
