// Copyright 2016 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.
//
//go:build ppc64 || s390x || mips || mips64
// +build ppc64 s390x mips mips64

package syscall

const isBigEndian = true
