// errorcheckdir

// Copyright 2021 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Issue 43962: Importing a package called "init" is an error.

package ignored
