// compiledir

// Copyright 2016 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// Issue 16317: cmd/compile: internal compiler error:
//              unhandled OCONV INT -> TUNSAFEPTR

package ignored
